@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -293,11 +293,14 @@ namespace Implementation {
 
 /* Generic types where result type is the same as value type */
 template<class V> struct ResultTraits {
-    typedef V Type;
+    typedef typename std::remove_const<V>::type Type;
 };
 template<class T> struct ResultTraits<Math::CubicHermite<T>> {
     typedef T Type;
 };
+template<class T> struct ResultTraits<const Math::CubicHermite<T>> {
+    typedef T Type;
+};
 template<class V> struct TypeTraits<V, V> {
     typedef V(*Interpolator)(const V&, const V&, Float);
 
@@ -312,7 +315,7 @@ template<class V> auto TypeTraits<V, V>::interpolator(Interpolation interpolatio
         case Interpolation::Custom: ; /* nope */
     }
 
-    CORRADE_ASSERT(false, "Animation::interpolatorFor(): can't deduce interpolator function for" << interpolation, {});
+    CORRADE_ASSERT_UNREACHABLE("Animation::interpolatorFor(): can't deduce interpolator function for" << interpolation, {});
 }
 
 /* Specialization for booleans (no linear interpolation) */
@@ -330,42 +333,86 @@ template<class T> auto TypeTraitsBool<T>::interpolator(Interpolation interpolati
         case Interpolation::Custom: ; /* nope */
     }
 
-    CORRADE_ASSERT(false, "Animation::interpolatorFor(): can't deduce interpolator function for" << interpolation, {});
+    CORRADE_ASSERT_UNREACHABLE("Animation::interpolatorFor(): can't deduce interpolator function for" << interpolation, {});
 }
 template<> struct TypeTraits<bool, bool>: TypeTraitsBool<bool> {};
 template<std::size_t size> struct TypeTraits<Math::BoolVector<size>, Math::BoolVector<size>>: TypeTraitsBool<Math::BoolVector<size>> {};
 
 /* Complex, preferring slerp() as it is more precise */
-template<class T> struct MAGNUM_EXPORT TypeTraits<Math::Complex<T>, Math::Complex<T>> {
+template<class T> struct
+#ifndef CORRADE_TARGET_CLANG_CL
+/* Clang-CL complains that it's ignored if it's on the class, so putting it
+   on the function instead. However MSVC doesn't like that, so doing this only
+   for Clang-CL. */
+MAGNUM_EXPORT
+#endif
+TypeTraits<Math::Complex<T>, Math::Complex<T>> {
     typedef Math::Complex<T>(*Interpolator)(const Math::Complex<T>&, const Math::Complex<T>&, Float);
 
-    static Interpolator interpolator(Interpolation interpolation);
+    static
+    #ifdef CORRADE_TARGET_CLANG_CL
+    MAGNUM_EXPORT
+    #endif
+    Interpolator interpolator(Interpolation interpolation);
 };
 
 /* Quaternions and dual quaternions, preferring slerp() as it is more precise */
-template<class T> struct MAGNUM_EXPORT TypeTraits<Math::Quaternion<T>, Math::Quaternion<T>> {
+template<class T> struct
+#ifndef CORRADE_TARGET_CLANG_CL
+/* Clang-CL complains that it's ignored if it's on the class, so putting it
+   on the function instead. However MSVC doesn't like that, so doing this only
+   for Clang-CL. */
+MAGNUM_EXPORT
+#endif
+TypeTraits<Math::Quaternion<T>, Math::Quaternion<T>> {
     typedef Math::Quaternion<T>(*Interpolator)(const Math::Quaternion<T>&, const Math::Quaternion<T>&, Float);
 
-    static Interpolator interpolator(Interpolation interpolation);
+    static
+    #ifdef CORRADE_TARGET_CLANG_CL
+    MAGNUM_EXPORT
+    #endif
+    Interpolator interpolator(Interpolation interpolation);
 };
-template<class T> struct MAGNUM_EXPORT TypeTraits<Math::DualQuaternion<T>, Math::DualQuaternion<T>> {
+template<class T> struct
+#ifndef CORRADE_TARGET_CLANG_CL
+/* Clang-CL complains that it's ignored if it's on the class, so putting it
+   on the function instead. However MSVC doesn't like that, so doing this only
+   for Clang-CL. */
+MAGNUM_EXPORT
+#endif
+TypeTraits<Math::DualQuaternion<T>, Math::DualQuaternion<T>> {
     typedef Math::DualQuaternion<T>(*Interpolator)(const Math::DualQuaternion<T>&, const Math::DualQuaternion<T>&, Float);
 
-    static Interpolator interpolator(Interpolation interpolation);
+    static
+    #ifdef CORRADE_TARGET_CLANG_CL
+    MAGNUM_EXPORT
+    #endif
+    Interpolator interpolator(Interpolation interpolation);
 };
 
 /* Cubic Hermite spline point has a different result type */
-template<class T> struct MAGNUM_EXPORT TypeTraits<Math::CubicHermite<T>, T> {
+template<class T> struct
+#ifndef CORRADE_TARGET_CLANG_CL
+/* Clang-CL complains that it's ignored if it's on the class, so putting it
+   on the function instead. However MSVC doesn't like that, so doing this only
+   for Clang-CL. */
+MAGNUM_EXPORT
+#endif
+TypeTraits<Math::CubicHermite<T>, T> {
     typedef T(*Interpolator)(const Math::CubicHermite<T>&, const Math::CubicHermite<T>&, Float);
 
-    static Interpolator interpolator(Interpolation interpolation);
+    static
+    #ifdef CORRADE_TARGET_CLANG_CL
+    MAGNUM_EXPORT
+    #endif
+    Interpolator interpolator(Interpolation interpolation);
 };
 
 }
 
 /* Needs to be defined later so it can pick up the TypeTraits definitions */
 template<class V, class R> auto interpolatorFor(Interpolation interpolation) -> R(*)(const V&, const V&, Float) {
-    return Implementation::TypeTraits<V, R>::interpolator(interpolation);
+    return Implementation::TypeTraits<typename std::remove_const<V>::type, R>::interpolator(interpolation);
 }
 
 template<class K, class V, class R> R interpolate(const Containers::StridedArrayView1D<const K>& keys, const Containers::StridedArrayView1D<const V>& values, const Extrapolation before, const Extrapolation after, R(*const interpolator)(const V&, const V&, Float), K frame, std::size_t& hint) {
