@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -115,23 +115,6 @@ template<UnsignedInt dimensions> class BufferImage {
          */
         explicit BufferImage(PixelFormat format, PixelType type, const VectorTypeFor<dimensions, Int>& size, Containers::ArrayView<const void> data, BufferUsage usage): BufferImage{{}, format, type, size, data, usage} {}
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /**
-         * @brief Constructor
-         * @deprecated Use either @ref GL::PixelFormat together with
-         *      @ref GL::PixelType or just @ref Magnum::PixelFormat instead
-         */
-        explicit CORRADE_DEPRECATED("use either GL::PixelFormat together with GL::PixelType or just Magnum::PixelFormat instead") BufferImage(PixelStorage storage, Magnum::PixelFormat format, PixelType type, const VectorTypeFor<dimensions, Int>& size, Containers::ArrayView<const void> data, BufferUsage usage): BufferImage{storage, PixelFormat(UnsignedInt(format)), type, size, data, usage} {}
-
-        /**
-         * @brief Constructor
-         *
-         * @deprecated Use either @ref GL::PixelFormat together with
-         *      @ref GL::PixelType or just @ref Magnum::PixelFormat instead
-         */
-        explicit CORRADE_DEPRECATED("use either GL::PixelFormat together with GL::PixelType or just Magnum::PixelFormat instead") BufferImage(Magnum::PixelFormat format, PixelType type, const VectorTypeFor<dimensions, Int>& size, Containers::ArrayView<const void> data, BufferUsage usage): BufferImage{PixelFormat(UnsignedInt(format)), type, size, data, usage} {}
-        #endif
-
         /**
          * @brief Constructor
          * @param storage           Storage of pixel data
@@ -185,22 +168,6 @@ template<UnsignedInt dimensions> class BufferImage {
          */
         explicit BufferImage(PixelFormat format, PixelType type, const VectorTypeFor<dimensions, Int>& size, Buffer&& buffer, std::size_t dataSize) noexcept: BufferImage{{}, format, type, size, std::move(buffer), dataSize} {}
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /**
-         * @brief Construct from existing buffer
-         * @deprecated Use either @ref GL::PixelFormat together with
-         *      @ref GL::PixelType or just @ref Magnum::PixelFormat instead
-         */
-        explicit CORRADE_DEPRECATED("use either GL::PixelFormat together with GL::PixelType or just Magnum::PixelFormat instead") BufferImage(PixelStorage storage, Magnum::PixelFormat format, PixelType type, const VectorTypeFor<dimensions, Int>& size, Buffer&& buffer, std::size_t dataSize) noexcept: BufferImage{storage, PixelFormat(UnsignedInt(format)), type, size, std::move(buffer), dataSize} {}
-
-        /**
-         * @brief Construct from existing buffer
-         * @deprecated Use either @ref GL::PixelFormat together with
-         *      @ref GL::PixelType or just @ref Magnum::PixelFormat instead
-         */
-        explicit CORRADE_DEPRECATED("use either GL::PixelFormat together with GL::PixelType or just Magnum::PixelFormat instead") BufferImage(Magnum::PixelFormat format, PixelType type, const VectorTypeFor<dimensions, Int>& size, Buffer&& buffer, std::size_t dataSize) noexcept: BufferImage{PixelFormat(UnsignedInt(format)), type, size, std::move(buffer), dataSize} {}
-        #endif
-
         /**
          * @brief Construct from existing buffer
          * @param storage           Storage of pixel data
@@ -248,22 +215,6 @@ template<UnsignedInt dimensions> class BufferImage {
          */
         /*implicit*/ BufferImage(PixelFormat format, PixelType type): BufferImage{{}, format, type} {}
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /**
-         * @brief Construct an image placeholder
-         * @deprecated Use either @ref GL::PixelFormat together with
-         *      @ref GL::PixelType or just @ref Magnum::PixelFormat instead
-         */
-        /*implicit*/ CORRADE_DEPRECATED("use either GL::PixelFormat together with GL::PixelType or just Magnum::PixelFormat instead") BufferImage(PixelStorage storage, Magnum::PixelFormat format, PixelType type): BufferImage{storage, PixelFormat(UnsignedInt(format)), type} {}
-
-        /**
-         * @brief Construct an image placeholder
-         * @deprecated Use either @ref GL::PixelFormat together with
-         *      @ref GL::PixelType or just @ref Magnum::PixelFormat instead
-         */
-        /*implicit*/ CORRADE_DEPRECATED("use either GL::PixelFormat together with GL::PixelType or just Magnum::PixelFormat instead") BufferImage(Magnum::PixelFormat format, PixelType type): BufferImage{PixelFormat(UnsignedInt(format)), type} {}
-        #endif
-
         /**
          * @brief Construct an image placeholder
          * @param storage           Storage of pixel data
@@ -294,6 +245,8 @@ template<UnsignedInt dimensions> class BufferImage {
          *
          * This function can be safely used for constructing (and later
          * destructing) objects even without any OpenGL context being active.
+         * However note that this is a low-level and a potentially dangerous
+         * API, see the documentation of @ref NoCreate for alternatives.
          * @see @ref BufferImage(), @ref Buffer::wrap()
          */
         explicit BufferImage(NoCreateT) noexcept;
@@ -334,9 +287,7 @@ template<UnsignedInt dimensions> class BufferImage {
          *
          * See @ref PixelStorage::dataProperties() for more information.
          */
-        std::pair<VectorTypeFor<dimensions, std::size_t>, VectorTypeFor<dimensions, std::size_t>> dataProperties() const {
-            return Magnum::Implementation::imageDataProperties<dimensions>(*this);
-        }
+        std::pair<VectorTypeFor<dimensions, std::size_t>, VectorTypeFor<dimensions, std::size_t>> dataProperties() const;
 
         /** @brief Currently allocated data size */
         std::size_t dataSize() const { return _dataSize; }
@@ -371,26 +322,6 @@ template<UnsignedInt dimensions> class BufferImage {
             setData({}, format, type, size, data, usage);
         }
 
-        #ifdef MAGNUM_BUILD_DEPRECATED
-        /**
-         * @brief Set image data
-         * @deprecated Use either @ref GL::PixelFormat together with
-         *      @ref GL::PixelType or just @ref Magnum::PixelFormat instead
-         */
-        CORRADE_DEPRECATED("use either GL::PixelFormat together with GL::PixelType or just Magnum::PixelFormat instead") void setData(PixelStorage storage, Magnum::PixelFormat format, PixelType type, const VectorTypeFor<dimensions, Int>& size, Containers::ArrayView<const void> data, BufferUsage usage) {
-            setData(storage, PixelFormat(UnsignedInt(format)), type, size, data, usage);
-        }
-
-        /**
-         * @brief Set image data
-         * @deprecated Use either @ref GL::PixelFormat together with
-         *      @ref GL::PixelType or just @ref Magnum::PixelFormat instead
-         */
-        CORRADE_DEPRECATED("use either GL::PixelFormat together with GL::PixelType or just Magnum::PixelFormat instead") void setData(Magnum::PixelFormat format, PixelType type, const VectorTypeFor<dimensions, Int>& size, Containers::ArrayView<const void> data, BufferUsage usage) {
-            setData({}, PixelFormat(UnsignedInt(format)), type, size, data, usage);
-        }
-        #endif
-
         /**
          * @brief Set image data
          * @param storage           Storage of pixel data
@@ -632,6 +563,11 @@ template<UnsignedInt dimensions> class CompressedBufferImage {
          * The constructed instance is equivalent to moved-from state. Useful
          * in cases where you will overwrite the instance later anyway. Move
          * another object over it to make it useful.
+         *
+         * This function can be safely used for constructing (and later
+         * destructing) objects even without any OpenGL context being active.
+         * However note that this is a low-level and a potentially dangerous
+         * API, see the documentation of @ref NoCreate for alternatives.
          * @see @ref CompressedBufferImage(), @ref Buffer::wrap()
          */
         explicit CompressedBufferImage(NoCreateT) noexcept;
@@ -666,9 +602,7 @@ template<UnsignedInt dimensions> class CompressedBufferImage {
          * @requires_gl Compressed pixel storage is hardcoded in OpenGL ES and
          *      WebGL.
          */
-        std::pair<VectorTypeFor<dimensions, std::size_t>, VectorTypeFor<dimensions, std::size_t>> dataProperties() const {
-            return Magnum::Implementation::compressedImageDataProperties<dimensions>(*this);
-        }
+        std::pair<VectorTypeFor<dimensions, std::size_t>, VectorTypeFor<dimensions, std::size_t>> dataProperties() const;
 
         /**
          * @brief Image buffer
@@ -764,87 +698,7 @@ typedef CompressedBufferImage<2> CompressedBufferImage2D;
 /** @brief Three-dimensional compressed buffer image */
 typedef CompressedBufferImage<3> CompressedBufferImage3D;
 
-template<UnsignedInt dimensions> inline BufferImage<dimensions>::BufferImage(BufferImage<dimensions>&& other) noexcept: _storage{std::move(other._storage)}, _format{std::move(other._format)}, _type{std::move(other._type)}, _size{std::move(other._size)}, _buffer{std::move(other._buffer)}, _dataSize{std::move(other._dataSize)} {
-    other._size = {};
-}
-
-template<UnsignedInt dimensions> inline CompressedBufferImage<dimensions>::CompressedBufferImage(CompressedBufferImage<dimensions>&& other) noexcept: _storage{std::move(other._storage)}, _format{std::move(other._format)}, _size{std::move(other._size)}, _buffer{std::move(other._buffer)}, _dataSize{std::move(other._dataSize)} {
-    other._size = {};
-    other._dataSize = {};
-}
-
-template<UnsignedInt dimensions> inline BufferImage<dimensions>& BufferImage<dimensions>::operator=(BufferImage<dimensions>&& other) noexcept {
-    using std::swap;
-    swap(_storage, other._storage);
-    swap(_format, other._format);
-    swap(_type, other._type);
-    swap(_size, other._size);
-    swap(_buffer, other._buffer);
-    swap(_dataSize, other._dataSize);
-    return *this;
-}
-
-template<UnsignedInt dimensions> inline CompressedBufferImage<dimensions>& CompressedBufferImage<dimensions>::operator=(CompressedBufferImage<dimensions>&& other) noexcept {
-    using std::swap;
-    swap(_storage, other._storage);
-    swap(_format, other._format);
-    swap(_size, other._size);
-    swap(_buffer, other._buffer);
-    swap(_dataSize, other._dataSize);
-    return *this;
-}
-
-}
-
-#ifdef MAGNUM_BUILD_DEPRECATED
-/* Note: needs to be prefixed with Magnum:: otherwise Doxygen can't find it */
-
-/** @brief @copybrief GL::BufferImage
- * @deprecated Use @ref GL::BufferImage instead.
- */
-#ifndef CORRADE_MSVC2015_COMPATIBILITY /* Multiple definitions still broken */
-template<UnsignedInt dimensions> using BufferImage CORRADE_DEPRECATED_ALIAS("use GL::BufferImage instead") = Magnum::GL::BufferImage<dimensions>;
-#endif
-
-/** @brief @copybrief GL::BufferImage1D
- * @deprecated Use @ref GL::BufferImage1D instead.
- */
-typedef CORRADE_DEPRECATED("use GL::BufferImage1D instead") Magnum::GL::BufferImage1D BufferImage1D;
-
-/** @brief @copybrief GL::BufferImage2D
- * @deprecated Use @ref GL::BufferImage2D instead.
- */
-typedef CORRADE_DEPRECATED("use GL::BufferImage2D instead") Magnum::GL::BufferImage2D BufferImage2D;
-
-/** @brief @copybrief GL::BufferImage3D
- * @deprecated Use @ref GL::BufferImage3D instead.
- */
-typedef CORRADE_DEPRECATED("use GL::BufferImage3D instead") Magnum::GL::BufferImage3D BufferImage3D;
-
-/** @brief @copybrief GL::CompressedBufferImage
- * @deprecated Use @ref GL::CompressedBufferImage instead.
- */
-#ifndef CORRADE_MSVC2015_COMPATIBILITY /* Multiple definitions still broken */
-template<UnsignedInt dimensions> using CompressedBufferImage CORRADE_DEPRECATED_ALIAS("use GL::CompressedBufferImage instead") = Magnum::GL::BufferImage<dimensions>;
-#endif
-
-/** @brief @copybrief GL::CompressedBufferImage1D
- * @deprecated Use @ref GL::CompressedBufferImage1D instead.
- */
-typedef CORRADE_DEPRECATED("use GL::CompressedBufferImage1D instead") Magnum::GL::CompressedBufferImage1D CompressedBufferImage1D;
-
-/** @brief @copybrief GL::CompressedBufferImage2D
- * @deprecated Use @ref GL::CompressedBufferImage2D instead.
- */
-typedef CORRADE_DEPRECATED("use GL::CompressedBufferImage2D instead") Magnum::GL::CompressedBufferImage2D CompressedBufferImage2D;
-
-/** @brief @copybrief GL::CompressedBufferImage3D
- * @deprecated Use @ref GL::CompressedBufferImage3D instead.
- */
-typedef CORRADE_DEPRECATED("use GL::CompressedBufferImage3D instead") Magnum::GL::CompressedBufferImage3D CompressedBufferImage3D;
-#endif
-
-}
+}}
 #else
 #error this header is not available in OpenGL ES 2.0 build
 #endif
