@@ -3,8 +3,8 @@
 /*
     This file is part of Magnum.
 
-    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
-              Vladimír Vondruš <mosra@centrum.cz>
+    Copyright © 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+                2020 Vladimír Vondruš <mosra@centrum.cz>
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -29,6 +29,9 @@
  * @brief Class @ref Magnum::Platform::GlxApplication, macro @ref MAGNUM_GLXAPPLICATION_MAIN()
  */
 
+#include "Magnum/configure.h"
+
+#ifdef MAGNUM_TARGET_GL
 #include "Magnum/Platform/AbstractXApplication.h"
 #include "Magnum/Platform/Platform.h"
 
@@ -37,12 +40,11 @@ namespace Magnum { namespace Platform {
 /**
 @brief GLX application
 
-Application using pure X11 and GLX. Supports keyboard and mouse handling.
+@m_keywords{Application}
 
-This application library is available on desktop OpenGL and
-@ref MAGNUM_TARGET_DESKTOP_GLES "OpenGL ES emulation on desktop" on Linux. It
-depends on the **X11** library and is built if `WITH_GLXAPPLICATION` is enabled
-in CMake.
+Application using pure X11 and GLX. Supports keyboard and mouse handling.
+Available on desktop OpenGL and
+@ref MAGNUM_TARGET_DESKTOP_GLES "OpenGL ES emulation on desktop" on Linux.
 
 @section Platform-GlxApplication-bootstrap Bootstrap application
 
@@ -52,15 +54,24 @@ more information.
 
 @section Platform-GlxApplication-usage General usage
 
-In order to use this library from Cmake, you need to request the
-`GlxApplication` component of the `Magnum` package and link to the
-`Magnum::GlxApplication` target:
+This application library depends on the **X11** library and is built if `WITH_GLXAPPLICATION` is enabled when building Magnum. To use this library from
+CMake, request the `GlxApplication` component of the `Magnum` package and link
+to the `Magnum::GlxApplication` target:
 
 @code{.cmake}
 find_package(Magnum REQUIRED GlxApplication)
 
 # ...
-target_link_libraries(your-app Magnum::GlxApplication)
+target_link_libraries(your-app PRIVATE Magnum::GlxApplication)
+@endcode
+
+Additionally, if you're using Magnum as a CMake subproject, do the following
+* *before* calling @cmake find_package() @ce to ensure it's enabled, as the
+library is not built by default:
+
+@code{.cmake}
+set(WITH_GLXAPPLICATION ON CACHE BOOL "" FORCE)
+add_subdirectory(magnum EXCLUDE_FROM_ALL)
 @endcode
 
 If no other application is requested, you can also use the generic
@@ -82,6 +93,10 @@ MAGNUM_GLXAPPLICATION_MAIN(MyApplication)
 If no other application header is included, this class is also aliased to
 @cpp Platform::Application @ce and the macro is aliased to
 @cpp MAGNUM_APPLICATION_MAIN() @ce to simplify porting.
+
+@note This class is available only if Magnum is compiled with
+    @ref MAGNUM_TARGET_GL enabled (done by default). See @ref building-features
+    for more information.
 */
 class GlxApplication: public AbstractXApplication {
     public:
@@ -112,6 +127,8 @@ class GlxApplication: public AbstractXApplication {
 @brief Entry point for GLX-based applications
 @param className Class name
 
+@m_keywords{MAGNUM_APPLICATION_MAIN()}
+
 See @ref Magnum::Platform::GlxApplication "Platform::GlxApplication" for usage
 information. This macro abstracts out platform-specific entry point code and is
 equivalent to the following, see @ref portability-applications for more
@@ -145,5 +162,8 @@ typedef BasicScreenedApplication<GlxApplication> ScreenedApplication;
 #endif
 
 }}
+#else
+#error this header is available only in the OpenGL build
+#endif
 
 #endif
